/**
 * Advanced Internet Service Project
 * 
 * Speaker recognition enhanced voice conference.
 * 
 * Authors: Yancheng Li, Liang Wei, Zhaoyuan Zhang
 * 
 */

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.LineUnavailableException;

public class AudioClientDriver {
	private static final int DEFAULT_INTERNAL_BUFSIZ = AudioUtil.FRAME_SIZE;
	private static final int DEFAULT_EXTERNAL_BUFSIZ = AudioUtil.FRAME_SIZE;

	public static void main(String[] args) {
		if (args.length != 2) {
			System.out.println("Usage: java AudioClient server port");
			return;
		}

		String server = args[0];
		Socket socket = null;
		DataOutputStream out = null;

		int port = Integer.parseInt(args[1]);

		try {
			socket = new Socket(server, port);
			out = new DataOutputStream(socket.getOutputStream());
		} catch (UnknownHostException e) {
			System.err.println("Don't know about host: " + server);
			System.exit(0);
		} catch (IOException e) {
			System.err.println("Couldn't get I/O for " + "the connection to: "
					+ server);
			System.exit(0);
		}

		String strMixerName = null;
		float fFrameRate = 8000.0F;
		int nInternalBufferSize = DEFAULT_INTERNAL_BUFSIZ;
		int nExternalBufferSize = DEFAULT_EXTERNAL_BUFSIZ;

		AudioFormat audioFormat = new AudioFormat(
				AudioFormat.Encoding.PCM_SIGNED, fFrameRate, 16, 2, 4,
				fFrameRate, false);

		AudioClient audioLoop = null;
		try {
			audioLoop = new AudioClient(audioFormat, nInternalBufferSize,
					nExternalBufferSize, strMixerName, out);
		} catch (LineUnavailableException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		AudioClientReceiver clientReceiver = new AudioClientReceiver(socket, audioLoop);
		
		audioLoop.start();
		clientReceiver.start();
		
		ClientInterface clientInterface = new ClientInterface(audioLoop);
		clientInterface.setVisible(true);
	}
}
