/**
 * Advanced Internet Service Project
 * 
 * Speaker recognition enhanced voice conference.
 * 
 * Authors: Yancheng Li, Liang Wei, Zhaoyuan Zhang
 * 
 */

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;

import SpeakerRegonition.ISpeakerRecognizer;

public class AudioServer extends Thread {
	private int port;
	private ArrayList<DataOutputStream> senders;
	private ServerSocket welcomeSocket = null;
	ISpeakerRecognizer recognizer;
	private byte[] data;
	private int dataOffset;
	private boolean pending;

	public AudioServer(int port, ISpeakerRecognizer recognizer) {
		this.port = port;
		this.recognizer = recognizer;
		senders = new ArrayList<DataOutputStream>();
		data = new byte[AudioUtil.RECOGNIZATION_FRAME_SIZE];
		dataOffset = 0;
		pending = true;
		
	}

	public void write(byte[] content, int offset, int len) {
		if (content[0] == AudioUtil.START && content[1] == AudioUtil.START && content[2] == AudioUtil.START) {
			dataOffset = 0;
			pending = true;
			return;
		}
		
		try {
			for (int i = 0; i < senders.size(); i++) {
				try {
					senders.get(i).write(content, offset, len);
				} catch (SocketException e) {
					senders.get(i).close();
					senders.remove(i);
					break;
				}
			}
			
			if (pending) {
				for (int i = dataOffset, j = 0; i < AudioUtil.RECOGNIZATION_FRAME_SIZE && j < len; i++, j++) {
					data[i] = content[j];
					dataOffset++;
				}
				
				if (dataOffset >= AudioUtil.RECOGNIZATION_FRAME_SIZE) {
					String result = recognizer.recognize(data);
					System.out.println(result + " is speaking.");
					notifySpeaker(result);
					pending = false;					
					
				}
			}
			
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void notifySpeaker(String name) {
		byte[] result = new byte[AudioUtil.FRAME_SIZE];
		
		for (int i = 0; i < AudioUtil.FRAME_SIZE; i ++) {
			result[i] = 0;
		}
		
		result[0] = AudioUtil.NAME;
		result[1] = AudioUtil.NAME;
		result[2] = AudioUtil.NAME;
		
		for (int i = 0; i < name.length(); i++) {
			result[i + 3] = (byte) name.charAt(i);
		}
		
		try {
			for (int i = 0; i < senders.size(); i++) {
				try {
					senders.get(i).write(result, 0, AudioUtil.FRAME_SIZE);
				} catch (SocketException e) {
					senders.get(i).close();
					senders.remove(i);
					break;
				}
			} 
		}catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}

	public void run() {
		try {

			System.out
					.println("Vocie conference server start...listening on port "
							+ port);
			welcomeSocket = new ServerSocket(port);
			while (true) {

				Socket connectionSocket = welcomeSocket.accept();

				System.out.println("new connection");
				
				AudioServerReceiver serverReceiver = new AudioServerReceiver(
						connectionSocket, this);
				
				DataOutputStream out = 				
						new DataOutputStream(connectionSocket.getOutputStream());
				senders.add(out);				
				serverReceiver.start();
				Thread.sleep(100);

			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

}
