/**
 * Advanced Internet Service Project
 * 
 * Speaker recognition enhanced voice conference.
 * 
 * Authors: Yancheng Li, Liang Wei, Zhaoyuan Zhang
 * 
 */
import java.awt.*;
import java.awt.event.*;

import javax.sound.sampled.AudioFormat;
import javax.swing.*;

import SpeakerRegonition.WaveformGraph;

public class ClientInterface extends JFrame implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -5343159026561957258L;
	private JPanel grappanel, buttonpanel;
	private JButton recordbutton, stopbutton, trainButton, saveButton, cancelButton;
	public static WaveformGraph wavMap;
	private boolean isRecord;
	private AudioClient client;
	public String speakerName;

	ClientInterface(AudioClient client) {				
		
		setTitle("Speaker Recognition System Client");
		setSize(500, 300);
		this.client = client;
		client.setClientInterface(this);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		Container c = getContentPane();
		c.setLayout(new BorderLayout());

		grappanel = new JPanel();
		wavMap = new WaveformGraph();
		grappanel.setLayout(new BorderLayout());
		JScrollPane grapscroll = new JScrollPane(wavMap,
				JScrollPane.VERTICAL_SCROLLBAR_NEVER,
				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		grappanel.add(grapscroll, BorderLayout.CENTER);
		c.add(grappanel, BorderLayout.CENTER);

		buttonpanel = new JPanel();
		buttonpanel.setSize(500, 30);
		buttonpanel.setLayout(new GridLayout(1, 6));

		recordbutton = new JButton("Start");
		stopbutton = new JButton("Pause");
		trainButton = new JButton("Train");
		saveButton = new JButton("Save");
		cancelButton = new JButton("Cancel");
		
		stopbutton.setEnabled(false);
		saveButton.setEnabled(false);
		buttonpanel.add(recordbutton);
		buttonpanel.add(stopbutton);
		buttonpanel.add(trainButton);
		buttonpanel.add(saveButton);
		buttonpanel.add(cancelButton);
		//buttonpanel.add(speaker);

		c.add(buttonpanel, BorderLayout.SOUTH);		

		int ScreenWidth = getToolkit().getScreenSize().width;
		int ScreenHeight = getToolkit().getScreenSize().height;
		setLocation((ScreenWidth - getWidth()) / 2,
				(ScreenHeight - getHeight()) / 2);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		recordbutton.addActionListener(this);
		stopbutton.addActionListener(this);
		trainButton.addActionListener(this);
		saveButton.addActionListener(this);
		cancelButton.addActionListener(this);

		isRecord = false;
	}

	public void actionPerformed(ActionEvent be) {
		if (be.getActionCommand() == "Start") {
			isRecord = true;

			recordbutton.setEnabled(false);
			stopbutton.setEnabled(true);
			client.startRecording();
			
			setSpeaker("Start speaking");
			
		} else if (be.getActionCommand() == "Pause") {
			isRecord = false;
			recordbutton.setEnabled(true);
			stopbutton.setEnabled(false);
			client.stopRecording();
			setSpeaker("Pause speaking");
			
		} else if (be.getActionCommand() == "Train") {
			
			speakerName = JOptionPane.showInputDialog(this,
			"Please input your name");
			if (speakerName != null) {
				System.out.println(speakerName);
				saveButton.setEnabled(true);
				trainButton.setEnabled(false);
				client.startTrainging();
			}
			
		} else if (be.getActionCommand() == "Save") {
			saveButton.setEnabled(false);
			trainButton.setEnabled(true);
			
			
		} else if (be.getActionCommand() == "Cancel") {
			saveButton.setEnabled(false);
			trainButton.setEnabled(true);
			client.getTrainingData();
		}
		
	}
	
	public void updateWaveFom(double[] data, AudioFormat audioFormat) {
		wavMap.doCreateWaveForm(data, audioFormat);
	}
	
	public void setSpeaker(String text) {
		setTitle(text);
	}
}
