/**
 * Advanced Internet Service Project
 * 
 * Speaker recognition enhanced voice conference.
 * 
 * Authors: Yancheng Li, Liang Wei, Zhaoyuan Zhang
 * 
 */

package SpeakerRegonition;

public class FeatureExtractor {
	private int framesCount;
	private double[][] lpc;
	private double[][] lpcc;
	private double[] pitch;
	private double[][] overlappedLpc;
	private double[] overlappedPitch;
	public double[][] featureVector;

	public FeatureExtractor(double[][] frame) {
		framesCount = frame.length;
		
		lpc = FeatureExtractorUtils.calculateFrameLPC(frame, framesCount);
		lpcc = FeatureExtractorUtils.calculateFrameLPCC(lpc, framesCount);
		pitch = FeatureExtractorUtils.detectPitch(frame, framesCount);
		
		overlappedLpc = new double[framesCount][];
		for (int i = 0; i < framesCount; i++)
			overlappedLpc[i] = FeatureExtractorUtils.getOverlappedData(lpcc[i]);
		
		overlappedPitch = FeatureExtractorUtils.getOverlappedData(pitch);
		
		calculateFrameFeature();
	}	

	private void calculateFrameFeature() {

		if (framesCount - 4 > 0)
			featureVector = new double[framesCount - 4][];
		else
			featureVector = new double[0][];

		for (int i = 2; i < framesCount - 2; i++) {
			featureVector[i - 2] = new double[1 + 1 + lpcc[i].length
					+ overlappedLpc[i - 2].length];
			
			featureVector[i - 2][0] = pitch[i];
			
			featureVector[i - 2][1] = overlappedPitch[i - 2];
			
			for (int j = 0; j < lpcc[i].length; j++)
				featureVector[i - 2][2 + j] = 2000 * lpcc[i][j];
			
			for (int j = 0; j < overlappedLpc[i - 2].length; j++)
				featureVector[i - 2][2 + lpcc[i].length + j] = 1000 * overlappedLpc[i - 2][j];
		}
	}
}
